////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>
#include <WinLDAP.H> // Requires WLDAP32.Lib

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CCmdLine.H"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CCmdLine::CCmdLine()
{
    sArgs = NULL;
    iArgCount = NULL;
    iArgLens = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

CCmdLine::~CCmdLine()
{
    sArgs = NULL;
    iArgCount = NULL;
    iArgLens = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCmdLine::Init(void)
{
    LPTSTR CommandLine = GetCommandLine();

    WCHAR *wcCmdLine = NULL;
    int CommandLineLen = strlen(CommandLine);

    if((wcCmdLine = (WCHAR *) calloc(sizeof(WCHAR), CommandLineLen + 1)) == NULL)
    {
        return false;
    }

    LdapUTF8ToUnicode(CommandLine, CommandLineLen, wcCmdLine, CommandLineLen + 1);

    LPWSTR *wcsArgs = CommandLineToArgvW(wcCmdLine, &iArgCount);

    free(wcCmdLine);

    if( (sArgs = (char **) calloc(sizeof(char *), iArgCount)) == NULL)
    {
        GlobalFree(wcsArgs);
        return false;
    }

    if( (iArgLens = (int *) calloc(sizeof(int *), iArgCount)) == NULL)
    {
        GlobalFree(wcsArgs);
        free(sArgs);
        return false;
    }

    int ArgLoop = 0;
    while(ArgLoop < iArgCount)
    {
        iArgLens[ArgLoop] = wcslen(wcsArgs[ArgLoop]);
        if((sArgs[ArgLoop] = (char *) calloc(sizeof(char), iArgLens[ArgLoop] + 1)) == NULL)
        {
            GlobalFree(wcsArgs);
            Destroy();
            return false;
        }
        LdapUnicodeToUTF8(wcsArgs[ArgLoop], iArgLens[ArgLoop], sArgs[ArgLoop], iArgLens[ArgLoop]);
        sArgs[ArgLoop][iArgLens[ArgLoop]] = '\0';
        ArgLoop++;
    }

    GlobalFree(wcsArgs);

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CCmdLine::Destroy(void)
{
    int ArgLoop = 0;

    while(ArgLoop < iArgCount)
    {
        free(sArgs[ArgLoop]);
        sArgs[ArgLoop] = NULL;
        iArgLens[ArgLoop] = 0;
        ArgLoop++;
    }

    free(iArgLens);
    free(sArgs);

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
